@@ -1,21 +1,18 @@
-# Copyright (C) 2018-2019  The Software Heritage developers
+# Copyright (C) 2018-2020  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
-import click
-import dulwich.repo
 import os
 import sys
+from typing import Dict, List, Optional
 
-from functools import partial
-from urllib.parse import urlparse
-
-from swh.model import hashutil
-from swh.model import identifiers as pids
-from swh.model.exceptions import ValidationError
-from swh.model.from_disk import Content, Directory
+# WARNING: do not import unnecessary things here to keep cli startup time under
+# control
+import click
 
+from swh.core.cli import swh as swh_cli_group
+from swh.model.identifiers import CoreSWHID, ObjectType
 
 CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
 
@@ -29,43 +26,84 @@ _DULWICH_TYPES = {
 }
 
 
-class PidParamType(click.ParamType):
-    name = "persistent identifier"
+class CoreSWHIDParamType(click.ParamType):
+    """Click argument that accepts a core SWHID and returns them as
+    :class:`swh.model.identifiers.CoreSWHID` instances """
+
+    name = "SWHID"
+
+    def convert(self, value, param, ctx) -> CoreSWHID:
+        from swh.model.exceptions import ValidationError
 
-    def convert(self, value, param, ctx):
         try:
-            pids.parse_persistent_identifier(value)
-            return value  # return as string, as we need just that
+            return CoreSWHID.from_string(value)
         except ValidationError as e:
-            self.fail("%s is not a valid SWHID. %s." % (value, e), param, ctx)
+            self.fail(f'"{value}" is not a valid core SWHID: {e}', param, ctx)
 
 
-def pid_of_file(path):
+def swhid_of_file(path) -> CoreSWHID:
+    from swh.model.from_disk import Content
+    from swh.model.hashutil import hash_to_bytes
+
     object = Content.from_file(path=path).get_data()
-    return pids.persistent_identifier(pids.CONTENT, object)
-
-
-def pid_of_file_content(data):
-    object = Content.from_bytes(mode=644, data=data).get_data()
-    return pids.persistent_identifier(pids.CONTENT, object)
-
-
-def pid_of_dir(path):
-    object = Directory.from_disk(path=path).get_data()
-    return pids.persistent_identifier(pids.DIRECTORY, object)
-
-
-def pid_of_origin(url):
-    pid = pids.PersistentId(
-        object_type="origin", object_id=pids.origin_identifier({"url": url})
+    return CoreSWHID(
+        object_type=ObjectType.CONTENT, object_id=hash_to_bytes(object["sha1_git"])
     )
-    return str(pid)
 
 
-def pid_of_git_repo(path):
+def swhid_of_file_content(data) -> CoreSWHID:
+    from swh.model.from_disk import Content
+    from swh.model.hashutil import hash_to_bytes
+
+    object = Content.from_bytes(mode=644, data=data).get_data()
+    return CoreSWHID(
+        object_type=ObjectType.CONTENT, object_id=hash_to_bytes(object["sha1_git"])
+    )
+
+
+def swhid_of_dir(path: bytes, exclude_patterns: List[bytes] = None) -> CoreSWHID:
+    from swh.model.from_disk import (
+        Directory,
+        accept_all_directories,
+        ignore_directories_patterns,
+    )
+    from swh.model.hashutil import hash_to_bytes
+
+    dir_filter = (
+        ignore_directories_patterns(path, exclude_patterns)
+        if exclude_patterns
+        else accept_all_directories
+    )
+
+    object = Directory.from_disk(path=path, dir_filter=dir_filter).get_data()
+    return CoreSWHID(
+        object_type=ObjectType.DIRECTORY, object_id=hash_to_bytes(object["id"])
+    )
+
+
+def swhid_of_origin(url):
+    from swh.model.hashutil import hash_to_bytes
+    from swh.model.identifiers import (
+        ExtendedObjectType,
+        ExtendedSWHID,
+        origin_identifier,
+    )
+
+    return ExtendedSWHID(
+        object_type=ExtendedObjectType.ORIGIN,
+        object_id=hash_to_bytes(origin_identifier({"url": url})),
+    )
+
+
+def swhid_of_git_repo(path) -> CoreSWHID:
+    import dulwich.repo
+
+    from swh.model import hashutil
+    from swh.model.identifiers import snapshot_identifier
+
     repo = dulwich.repo.Repo(path)
 
-    branches = {}
+    branches: Dict[bytes, Optional[Dict]] = {}
     for ref, target in repo.refs.as_dict().items():
         obj = repo[target]
         if obj:
@@ -84,13 +122,15 @@ def pid_of_git_repo(path):
 
     snapshot = {"branches": branches}
 
-    pid = pids.PersistentId(
-        object_type="snapshot", object_id=pids.snapshot_identifier(snapshot)
+    return CoreSWHID(
+        object_type=ObjectType.SNAPSHOT,
+        object_id=hashutil.hash_to_bytes(snapshot_identifier(snapshot)),
     )
-    return str(pid)
 
 
-def identify_object(obj_type, follow_symlinks, obj):
+def identify_object(obj_type, follow_symlinks, exclude_patterns, obj) -> str:
+    from urllib.parse import urlparse
+
     if obj_type == "auto":
         if obj == "-" or os.path.isfile(obj):
             obj_type = "content"
@@ -105,32 +145,32 @@ def identify_object(obj_type, follow_symlinks, obj):
             except ValueError:
                 raise click.BadParameter("cannot detect object type for %s" % obj)
 
-    pid = None
-
     if obj == "-":
         content = sys.stdin.buffer.read()
-        pid = pid_of_file_content(content)
+        swhid = str(swhid_of_file_content(content))
     elif obj_type in ["content", "directory"]:
         path = obj.encode(sys.getfilesystemencoding())
         if follow_symlinks and os.path.islink(obj):
             path = os.path.realpath(obj)
         if obj_type == "content":
-            pid = pid_of_file(path)
+            swhid = str(swhid_of_file(path))
         elif obj_type == "directory":
-            pid = pid_of_dir(path)
+            swhid = str(
+                swhid_of_dir(path, [pattern.encode() for pattern in exclude_patterns])
+            )
     elif obj_type == "origin":
-        pid = pid_of_origin(obj)
+        swhid = str(swhid_of_origin(obj))
     elif obj_type == "snapshot":
-        pid = pid_of_git_repo(obj)
+        swhid = str(swhid_of_git_repo(obj))
     else:  # shouldn't happen, due to option validation
         raise click.BadParameter("invalid object type: " + obj_type)
 
     # note: we return original obj instead of path here, to preserve user-given
     # file name in output
-    return (obj, pid)
+    return swhid
 
 
-@click.command(context_settings=CONTEXT_SETTINGS)
+@swh_cli_group.command(context_settings=CONTEXT_SETTINGS)
 @click.option(
     "--dereference/--no-dereference",
     "follow_symlinks",
@@ -152,15 +192,26 @@ def identify_object(obj_type, follow_symlinks, obj):
     type=click.Choice(["auto", "content", "directory", "origin", "snapshot"]),
     help="type of object to identify (default: auto)",
 )
+@click.option(
+    "--exclude",
+    "-x",
+    "exclude_patterns",
+    metavar="PATTERN",
+    multiple=True,
+    help="Exclude directories using glob patterns \
+    (e.g., '*.git' to exclude all .git directories)",
+)
 @click.option(
     "--verify",
     "-v",
     metavar="SWHID",
-    type=PidParamType(),
+    type=CoreSWHIDParamType(),
     help="reference identifier to be compared with computed one",
 )
 @click.argument("objects", nargs=-1, required=True)
-def identify(obj_type, verify, show_filename, follow_symlinks, objects):
+def identify(
+    obj_type, verify, show_filename, follow_symlinks, objects, exclude_patterns,
+):
     """Compute the Software Heritage persistent identifier (SWHID) for the given
     source code object(s).
 
@@ -190,25 +241,32 @@ def identify(obj_type, verify, show_filename, follow_symlinks, objects):
       swh:1:snp:510aa88bdc517345d258c1fc2babcd0e1f905e93	helloworld.git
 
     """  # NoQA  # overlong lines in shell examples are fine
+    from functools import partial
 
     if verify and len(objects) != 1:
         raise click.BadParameter("verification requires a single object")
 
-    results = map(partial(identify_object, obj_type, follow_symlinks), objects)
+    results = zip(
+        objects,
+        map(
+            partial(identify_object, obj_type, follow_symlinks, exclude_patterns),
+            objects,
+        ),
+    )
 
     if verify:
-        pid = next(results)[1]
-        if verify == pid:
-            click.echo("SWHID match: %s" % pid)
+        swhid = next(results)[1]
+        if str(verify) == swhid:
+            click.echo("SWHID match: %s" % swhid)
             sys.exit(0)
         else:
-            click.echo("SWHID mismatch: %s != %s" % (verify, pid))
+            click.echo("SWHID mismatch: %s != %s" % (verify, swhid))
             sys.exit(1)
     else:
-        for (obj, pid) in results:
-            msg = pid
+        for (obj, swhid) in results:
+            msg = swhid
             if show_filename:
-                msg = "%s\t%s" % (pid, os.fsdecode(obj))
+                msg = "%s\t%s" % (swhid, os.fsdecode(obj))
             click.echo(msg)
 
 
