@@ -3,30 +3,60 @@
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
+import datetime
 import enum
+import fnmatch
+import glob
 import os
+import re
 import stat
+from typing import Any, Iterable, Iterator, List, Optional, Pattern, Tuple
 
 import attr
-from typing import List, Optional, Iterable, Any
+from attrs_strict import type_validator
+from typing_extensions import Final
 
-from .hashutil import MultiHash
-from .merkle import MerkleLeaf, MerkleNode
-from .identifiers import (
-    directory_entry_sort_key,
-    directory_identifier,
-    identifier_to_bytes as id_to_bytes,
-    identifier_to_str as id_to_str,
-)
 from . import model
+from .exceptions import InvalidDirectoryPath
+from .hashutil import MultiHash
+from .identifiers import directory_entry_sort_key, directory_identifier
+from .identifiers import identifier_to_bytes as id_to_bytes
+from .identifiers import identifier_to_str as id_to_str
+from .merkle import MerkleLeaf, MerkleNode
 
 
-@attr.s
-class DiskBackedContent(model.Content):
-    """Subclass of Content, which allows lazy-loading data from the disk."""
+@attr.s(frozen=True, slots=True)
+class DiskBackedContent(model.BaseContent):
+    """Content-like class, which allows lazy-loading data from the disk."""
+
+    object_type: Final = "content_file"
+
+    sha1 = attr.ib(type=bytes, validator=type_validator())
+    sha1_git = attr.ib(type=model.Sha1Git, validator=type_validator())
+    sha256 = attr.ib(type=bytes, validator=type_validator())
+    blake2s256 = attr.ib(type=bytes, validator=type_validator())
+
+    length = attr.ib(type=int, validator=type_validator())
+
+    status = attr.ib(
+        type=str,
+        validator=attr.validators.in_(["visible", "hidden"]),
+        default="visible",
+    )
+
+    ctime = attr.ib(
+        type=Optional[datetime.datetime],
+        validator=type_validator(),
+        default=None,
+        eq=False,
+    )
 
     path = attr.ib(type=Optional[bytes], default=None)
 
+    @classmethod
+    def from_dict(cls, d):
+        return cls(**d)
+
     def __attrs_post_init__(self):
         if self.path is None:
             raise TypeError("path must not be None.")
@@ -92,7 +122,7 @@ class Content(MerkleLeaf):
     """
 
     __slots__ = []  # type: List[str]
-    type = "content"
+    object_type: Final = "content"
 
     @classmethod
     def from_bytes(cls, *, mode, data):
@@ -250,6 +280,97 @@ def ignore_named_directories(names, *, case_sensitive=True):
     return named_filter
 
 
+# TODO: `extract_regex_objs` has been copied and adapted from `swh.scanner`.
+# In the future `swh.scanner` should use the `swh.model` version and remove its own.
+def extract_regex_objs(
+    root_path: bytes, patterns: Iterable[bytes]
+) -> Iterator[Pattern[bytes]]:
+    """Generates a regex object for each pattern given in input and checks if
+       the path is a subdirectory or relative to the root path.
+
+      Args:
+        root_path (bytes): path to the root directory
+        patterns (list of byte): patterns to match
+
+       Yields:
+          an SRE_Pattern object
+    """
+    absolute_root_path = os.path.abspath(root_path)
+    for pattern in patterns:
+        for path in glob.glob(pattern):
+            absolute_path = os.path.abspath(path)
+            if not absolute_path.startswith(absolute_root_path):
+                error_msg = (
+                    b'The path "' + path + b'" is not a subdirectory or relative '
+                    b'to the root directory path: "' + root_path + b'"'
+                )
+                raise InvalidDirectoryPath(error_msg)
+
+        regex = fnmatch.translate((pattern.decode()))
+        yield re.compile(regex.encode())
+
+
+def ignore_directories_patterns(root_path: bytes, patterns: Iterable[bytes]):
+    """Filter for :func:`directory_to_objects` to ignore directories
+    matching certain patterns.
+
+    Args:
+      root_path (bytes): path of the root directory
+      patterns (list of byte): patterns to ignore
+
+    Returns:
+      a directory filter for :func:`directory_to_objects`
+    """
+    sre_patterns = set(extract_regex_objs(root_path, patterns))
+
+    def pattern_filter(
+        dirpath: bytes,
+        dirname: bytes,
+        entries: Iterable[Any],
+        patterns: Iterable[Any] = sre_patterns,
+        root_path: bytes = os.path.abspath(root_path),
+    ):
+        full_path = os.path.abspath(dirpath)
+        relative_path = os.path.relpath(full_path, root_path)
+        return not any([pattern.match(relative_path) for pattern in patterns])
+
+    return pattern_filter
+
+
+def iter_directory(
+    directory,
+) -> Tuple[List[model.Content], List[model.SkippedContent], List[model.Directory]]:
+    """Return the directory listing from a disk-memory directory instance.
+
+    Raises:
+        TypeError in case an unexpected object type is listed.
+
+    Returns:
+        Tuple of respectively iterable of content, skipped content and directories.
+
+    """
+    contents: List[model.Content] = []
+    skipped_contents: List[model.SkippedContent] = []
+    directories: List[model.Directory] = []
+
+    for obj in directory.iter_tree():
+        obj = obj.to_model()
+        obj_type = obj.object_type
+        if obj_type in (model.Content.object_type, DiskBackedContent.object_type):
+            # FIXME: read the data from disk later (when the
+            # storage buffer is flushed).
+            obj = obj.with_data()
+            contents.append(obj)
+        elif obj_type == model.SkippedContent.object_type:
+            skipped_contents.append(obj)
+        elif obj_type == model.Directory.object_type:
+            directories.append(obj)
+        else:
+            raise TypeError(f"Unexpected object type from disk: {obj}")
+
+    return contents, skipped_contents, directories
+
+
 class Directory(MerkleNode):
     """Representation of a Software Heritage directory as a node in a Merkle Tree.
 
@@ -271,7 +392,7 @@ class Directory(MerkleNode):
     """
 
     __slots__ = ["__entries"]
-    type = "directory"
+    object_type: Final = "directory"
 
     @classmethod
     def from_disk(
@@ -323,14 +444,14 @@ class Directory(MerkleNode):
 
     @staticmethod
     def child_to_directory_entry(name, child):
-        if isinstance(child, Directory):
+        if child.object_type == "directory":
             return {
                 "type": "dir",
                 "perms": DentryPerms.directory,
                 "target": child.hash,
                 "name": name,
             }
-        elif isinstance(child, Content):
+        elif child.object_type == "content":
             return {
                 "type": "file",
                 "perms": child.data["perms"],
@@ -338,7 +459,7 @@ class Directory(MerkleNode):
                 "name": name,
             }
         else:
-            raise ValueError("unknown child")
+            raise ValueError(f"unknown child {child}")
 
     def get_data(self, **kwargs):
         return {
