@@ -24,31 +24,31 @@ from hypothesis.strategies import (
     sampled_from,
     sets,
     text,
+    tuples,
 )
 
 from .from_disk import DentryPerms
+from .identifiers import identifier_to_bytes, snapshot_identifier
 from .model import (
-    Person,
-    Timestamp,
-    TimestampWithTimezone,
+    BaseContent,
+    Content,
+    Directory,
+    DirectoryEntry,
+    ObjectType,
     Origin,
     OriginVisit,
     OriginVisitStatus,
-    Snapshot,
-    SnapshotBranch,
-    ObjectType,
-    TargetType,
+    Person,
     Release,
     Revision,
     RevisionType,
-    BaseContent,
-    Directory,
-    DirectoryEntry,
-    Content,
     SkippedContent,
+    Snapshot,
+    SnapshotBranch,
+    TargetType,
+    Timestamp,
+    TimestampWithTimezone,
 )
-from .identifiers import snapshot_identifier, identifier_to_bytes
-
 
 pgsql_alphabet = characters(
     blacklist_categories=("Cs",), blacklist_characters=["\u0000"]
@@ -72,7 +72,17 @@ def sha1():
 
 
 def aware_datetimes():
-    return datetimes(timezones=timezones())
+    # datetimes in Software Heritage are not used for software artifacts
+    # (which may be much older than 2000), but only for objects like scheduler
+    # task runs, and origin visits, which were created by Software Heritage,
+    # so at least in 2015.
+    # We're forbidding old datetimes, because until 1956, many timezones had seconds
+    # in their "UTC offsets" (see
+    # <https://en.wikipedia.org/wiki/Time_zone#Worldwide_time_zones>), which is not
+    # encodable in ISO8601; and we need our datetimes to be ISO8601-encodable in the
+    # RPC protocol
+    min_value = datetime.datetime(2000, 1, 1, 0, 0, 0)
+    return datetimes(min_value=min_value, timezones=timezones())
 
 
 @composite
@@ -83,10 +93,13 @@ def urls(draw):
     return "%s://%s" % (protocol, domain)
 
 
-def persons_d():
-    return builds(
-        dict, fullname=binary(), email=optional(binary()), name=optional(binary()),
-    )
+@composite
+def persons_d(draw):
+    fullname = draw(binary())
+    email = draw(optional(binary()))
+    name = draw(optional(binary()))
+    assume(not (len(fullname) == 32 and email is None and name is None))
+    return dict(fullname=fullname, name=name, email=email)
 
 
 def persons():
@@ -141,12 +154,10 @@ def origins():
 def origin_visits_d():
     return builds(
         dict,
-        visit=integers(0, 1000),
+        visit=integers(1, 1000),
         origin=urls(),
         date=aware_datetimes(),
-        status=sampled_from(["ongoing", "full", "partial"]),
         type=pgsql_text(),
-        snapshot=optional(sha1_git()),
     )
 
 
@@ -161,12 +172,15 @@ def metadata_dicts():
 def origin_visit_statuses_d():
     return builds(
         dict,
-        visit=integers(0, 1000),
+        visit=integers(1, 1000),
         origin=urls(),
-        status=sampled_from(["ongoing", "full", "partial"]),
+        type=optional(sampled_from(["git", "svn", "pypi", "debian"])),
+        status=sampled_from(
+            ["created", "ongoing", "full", "partial", "not_found", "failed"]
+        ),
         date=aware_datetimes(),
         snapshot=optional(sha1_git()),
-        metadata=one_of(none(), metadata_dicts()),
+        metadata=optional(metadata_dicts()),
     )
 
 
@@ -178,10 +192,10 @@ def origin_visit_statuses():
 def releases_d(draw):
     target_type = sampled_from([x.value for x in ObjectType])
     name = binary()
-    message = binary()
+    message = optional(binary())
     synthetic = booleans()
     target = sha1_git()
-    metadata = one_of(none(), revision_metadata())
+    metadata = optional(revision_metadata())
 
     return draw(
         one_of(
@@ -218,19 +232,26 @@ def releases():
 revision_metadata = metadata_dicts
 
 
+def extra_headers():
+    return lists(
+        tuples(binary(min_size=0, max_size=50), binary(min_size=0, max_size=500))
+    ).map(tuple)
+
+
 def revisions_d():
     return builds(
         dict,
-        message=binary(),
+        message=optional(binary()),
         synthetic=booleans(),
         author=persons_d(),
         committer=persons_d(),
         date=timestamps_with_timezone_d(),
         committer_date=timestamps_with_timezone_d(),
-        parents=lists(sha1_git()),
+        parents=tuples(sha1_git()),
         directory=sha1_git(),
         type=sampled_from([x.value for x in RevisionType]),
-        metadata=one_of(none(), revision_metadata()),
+        metadata=optional(revision_metadata()),
+        extra_headers=extra_headers(),
     )
     # TODO: metadata['extra_headers'] can have binary keys and values
 
@@ -254,7 +275,7 @@ def directory_entries():
 
 
 def directories_d():
-    return builds(dict, entries=lists(directory_entries_d()))
+    return builds(dict, entries=tuples(directory_entries_d()))
 
 
 def directories():
@@ -337,7 +358,7 @@ def snapshots_d(draw, *, min_size=0, max_size=100, only_objects=False):
     branches = draw(
         dictionaries(
             keys=branch_names(),
-            values=one_of(none(), branch_targets_d(only_objects=only_objects)),
+            values=optional(branch_targets_d(only_objects=only_objects)),
             min_size=min_size,
             max_size=max_size,
         )
