@@ -1,4 +1,4 @@
-# Copyright (C) 2017 The Software Heritage developers
+# Copyright (C) 2017-2020 The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -6,9 +6,8 @@
 """Merkle tree data structure"""
 
 import abc
-import collections
-
-from typing import Iterator, List, Optional, Set
+from collections.abc import Mapping
+from typing import Iterator, List, Set
 
 
 def deep_update(left, right):
@@ -66,7 +65,7 @@ def deep_update(left, right):
 
     """
     for key, rvalue in right.items():
-        if isinstance(rvalue, collections.Mapping):
+        if isinstance(rvalue, Mapping):
             new_lvalue = deep_update(left.get(key, {}), rvalue)
             left[key] = new_lvalue
         else:
@@ -111,7 +110,6 @@ class MerkleNode(dict, metaclass=abc.ABCMeta):
 
     __slots__ = ["parents", "data", "__hash", "collected"]
 
-    type = None  # type: Optional[str]  # TODO: make this an enum
     """Type of the current node (used as a classifier for :func:`collect`)"""
 
     def __init__(self, data=None):
@@ -234,7 +232,7 @@ class MerkleNode(dict, metaclass=abc.ABCMeta):
         """
         if not self.collected:
             self.collected = True
-            return {self.type: {self.hash: self.get_data(**kwargs)}}
+            return {self.object_type: {self.hash: self.get_data(**kwargs)}}
         else:
             return {}
 
