@@ -1,4 +1,4 @@
-# Copyright (C) 2019 The Software Heritage developers
+# Copyright (C) 2019-2020 The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -7,19 +7,21 @@ import datetime
 
 import attr
 from hypothesis import given, settings
+import iso8601
 
 from swh.model.hashutil import DEFAULT_ALGORITHMS
 from swh.model.hypothesis_strategies import (
-    objects,
-    object_dicts,
+    aware_datetimes,
     contents,
+    object_dicts,
+    objects,
+    origin_visits,
+    persons,
     skipped_contents,
     snapshots,
-    origin_visits,
 )
 from swh.model.model import TargetType
 
-
 target_types = ("content", "directory", "revision", "release", "snapshot", "alias")
 all_but_skipped_content = (
     "origin",
@@ -63,7 +65,7 @@ def assert_nested_dict(obj):
         for (key, value) in obj.items():
             assert isinstance(key, (str, bytes)), key
             assert_nested_dict(value)
-    elif isinstance(obj, list):
+    elif isinstance(obj, tuple):
         for value in obj:
             assert_nested_dict(value)
     elif isinstance(obj, (int, float, str, bytes, bool, type(None), datetime.datetime)):
@@ -93,6 +95,14 @@ def test_dicts_generation(obj_type_and_obj):
             assert branch is None or branch["target_type"] in target_types
 
 
+@given(aware_datetimes())
+def test_datetimes(dt):
+    # Checks this doesn't raise an error, eg. about seconds in the TZ offset
+    iso8601.parse_date(dt.isoformat())
+
+    assert dt.tzinfo is not None
+
+
 @given(object_dicts(split_content=False))
 def test_dicts_generation_merged_content(obj_type_and_obj):
     # we should never generate a "skipped_content" here
@@ -112,11 +122,15 @@ def test_dicts_generation_blacklist(obj_type_and_obj):
 
 @given(objects())
 def test_model_to_dicts(obj_type_and_obj):
-    (obj_type, object_) = obj_type_and_obj
+    _, object_ = obj_type_and_obj
+    object_type = object_.object_type
     obj_dict = object_.to_dict()
     assert_nested_dict(obj_dict)
-    if obj_type == "content":
-        COMMON_KEYS = set(DEFAULT_ALGORITHMS) | {"length", "status", "ctime"}
+    if object_type in ("content", "skipped_content"):
+        COMMON_KEYS = set(DEFAULT_ALGORITHMS) | {"length", "status"}
+        if object_.ctime is not None:
+            COMMON_KEYS |= {"ctime"}
+
         if obj_dict["status"] == "visible":
             assert set(obj_dict) == COMMON_KEYS | {"data"}
         elif obj_dict["status"] == "absent":
@@ -125,9 +139,9 @@ def test_model_to_dicts(obj_type_and_obj):
             assert set(obj_dict) == COMMON_KEYS | {"data"}
         else:
             assert False, obj_dict
-    elif obj_type == "release":
+    elif object_type == "release":
         assert obj_dict["target_type"] in target_types
-    elif obj_type == "snapshot":
+    elif object_type == "snapshot":
         for branch in obj_dict["branches"].values():
             assert branch is None or branch["target_type"] in target_types
 
@@ -186,3 +200,10 @@ def test_snapshots_strategy_fixed_size(snapshot):
 @given(origin_visits())
 def test_origin_visit_aware_datetime(visit):
     assert visit.date.tzinfo is not None
+
+
+@given(persons())
+def test_person_do_not_look_like_anonimized(person):
+    assert not (
+        len(person.fullname) == 32 and person.name is None and person.email is None
+    )
